@@ -31,6 +31,7 @@
 
 #include <iostream>
 #include <string>
+#include <string_view>
 #include <vector>
 
 #include <apt-pkg/macros.h>
@@ -88,18 +89,19 @@ class APT_PUBLIC Configuration
    bool FindB(const char *Name,bool const &Default = false) const;
    bool FindB(std::string const &Name,bool const &Default = false) const {return FindB(Name.c_str(),Default);};
    std::string FindAny(const char *Name,const char *Default = 0) const;
-	      
-   inline void Set(const std::string &Name,const std::string &Value) {Set(Name.c_str(),Value);};
-   void CndSet(const char *Name,const std::string &Value);
+
+   inline void Set(const std::string &Name,const std::string_view &Value) {Set(Name.c_str(),Value);};
+   void CndSet(const char *Name,const std::string_view &Value);
    void CndSet(const char *Name,const int Value);
-   void Set(const char *Name,const std::string &Value);
+   void Set(const char *Name,const std::string_view &Value);
    void Set(const char *Name,const int &Value);
-   
+
    inline bool Exists(const std::string &Name) const {return Exists(Name.c_str());};
    bool Exists(const char *Name) const;
    bool ExistsAny(const char *Name) const;
 
    void MoveSubTree(char const * const OldRoot, char const * const NewRoot);
+   void MoveSubTree(char const *const OldRoot, char const *const NewRoot, bool Overwrite);
 
    // clear a whole tree
    void Clear(const std::string &Name);
@@ -116,6 +118,9 @@ class APT_PUBLIC Configuration
    void Dump(std::ostream& str, char const * const root,
 	     char const * const format, bool const emptyValue);
 
+#ifdef APT_COMPILING_APT
+   bool SectionInSubTree(char const *const SubTree, std::string_view Needle);
+#endif
    explicit Configuration(const Item *Root);
    Configuration();
    ~Configuration();
