@@ -104,12 +104,23 @@ bool pkgCacheGenerator::Start()
       map_stringitem_t idxArchitectures;
 
       std::vector<std::string> archs = APT::Configuration::getArchitectures();
-      if (archs.size() > 1)
+      auto const &variants = APT::Configuration::getArchitectureVariants(true);
+      if (archs.size() > 1 || not variants.empty())
       {
 	 std::vector<std::string>::const_iterator a = archs.begin();
 	 std::string list = *a;
 	 for (++a; a != archs.end(); ++a)
 	    list.append(",").append(*a);
+	 if (not variants.empty())
+	 {
+	    list.append(";");
+	    for (auto const &variant : variants)
+	    {
+	       if (list.back() != ';')
+		  list.append(",");
+	       list.append(variant.name);
+	    }
+	 }
 	 idxArchitectures = WriteStringInMap(list);
 	 if (unlikely(idxArchitectures == 0))
 	    return false;
@@ -251,6 +262,15 @@ bool pkgCacheGenerator::MergeList(ListParser &List,
       if (PackageName.empty() == true)
 	 return false;
 
+      // Package is excluded
+      if (exclude.find(PackageName) != exclude.end())
+	 continue;
+
+      // Includes are specified, this is the total set of packages to include,
+      // so we need to skip the package if it is not in the include list.
+      if (not include.empty() && include.find(PackageName) == include.end())
+	 continue;
+
       Counter++;
       if (Counter % 100 == 0 && Progress != 0)
 	 Progress->Progress(List.Offset());
@@ -374,11 +394,20 @@ bool pkgCacheGenerator::MergeListVersion(ListParser &List, pkgCache::PkgIterator
 
    auto Hash = List.VersionHash();
    std::string_view ListSHA256;
+   std::string_view ListArchVariant;
 
    bool const Debug = _config->FindB("Debug::pkgCacheGen", false);
    auto DebList = dynamic_cast<debListParser *>(&List);
    if (DebList != nullptr)
       ListSHA256 = DebList->SHA256();
+   if (DebList != nullptr)
+      ListArchVariant = DebList->ArchVariant();
+
+   auto variants = _config->FindVector("APT::Architecture-Variants");
+   // Always add "no variant" at the back of the list. If it's already configured earlier, that wins
+   if (variants.empty() || variants.back() != "")
+      variants.push_back("");
+
    if (Ver.end() == false)
    {
       /* We know the list is sorted so we use that fact in the search.
@@ -417,6 +446,13 @@ bool pkgCacheGenerator::MergeListVersion(ListParser &List, pkgCache::PkgIterator
 	       if (VF.end() == true)
 		  break;
 	    }
+
+	    // Variants are ordered by their index in the variant list.
+	    if (std::find(variants.begin(), variants.end(), Ver.ArchVariant()) > std::find(variants.begin(), variants.end(), ListArchVariant))
+	    {
+	       Res = 1;
+	       break;
+	    }
 	 }
 	 // proceed with the next till we have either the right
 	 // or we found another version (which will be lower)
@@ -757,7 +793,7 @@ bool pkgCacheGenerator::NewPackage(pkgCache::PkgIterator &Pkg, string_view Name,
 	       pkgCache::VerIterator V = Prv.OwnerVer();
 	       if (ArchA != V.ParentPkg().Arch())
 		  continue;
-	       if (NewProvides(V, Pkg, V->VerStr, pkgCache::Flag::MultiArchImplicit | pkgCache::Flag::ArchSpecific) == false)
+	       if (NewProvides(V, Pkg, Prv->ProvideVersion, pkgCache::Flag::MultiArchImplicit | pkgCache::Flag::ArchSpecific) == false)
 		  return false;
 	    }
 	    pkgCache::VerIterator V = PkgA.VersionList();
@@ -1332,6 +1368,7 @@ bool pkgCacheGenerator::SelectFile(std::string const &File,
 				   pkgIndexFile const &Index,
 				   std::string const &Architecture,
 				   std::string const &Component,
+				   const IndexTarget *Target,
 				   unsigned long const Flags)
 {
    CurrentFile = nullptr;
@@ -1374,6 +1411,22 @@ bool pkgCacheGenerator::SelectFile(std::string const &File,
    Cache.HeaderP->FileList = map_pointer<pkgCache::PackageFile>{NarrowOffset(CurrentFile - Cache.PkgFileP)};
    Cache.HeaderP->PackageFileCount++;
 
+   include.clear();
+   exclude.clear();
+   if (Target)
+   {
+      if (auto inc = Target->Options.find("INCLUDE"); inc != Target->Options.end())
+      {
+	 auto v = VectorizeString(inc->second, ' ');
+	 include.insert(v.begin(), v.end());
+      }
+      if (auto exc = Target->Options.find("EXCLUDE"); exc != Target->Options.end())
+      {
+	 auto v = VectorizeString(exc->second, ' ');
+	 exclude.insert(v.begin(), v.end());
+      }
+   }
+
    if (Progress != 0)
       Progress->SubProgress(Index.Size());
    return true;
